#!/usr/bin/perl -w
#
# Copyright (C) 2008 Patrick Fuerlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#//////////////////////////////////////////////////////////////////////////////
use FindBin qw ( $Bin );
use lib "$Bin/../lib/";
use strict;
use warnings;
use ReelBox::Base qw(vdr_status get_config);
use Storable;
use File::stat;
use Getopt::Long;
use LockFile::Simple qw(lock trylock unlock);
use File::Find;
use File::Basename;
use URI::Escape;
use Time::Local;
use LWP::Simple;
use File::Touch;
use IMDB::Film;
#//////////////////////////////////////////////////////////////////////////////
my $g_IP            = '192.168.0.176';
my %g_RbcConf       = %{ get_config('/opt/rbc/htdocs/config/reelbox.cfg') };
my $g_UpdateFlag    = "$g_RbcConf{'RecDir'}/.update";
my $g_RbcStatusHash = "$Bin/../var/rbc.hash";
my $g_RbcRecordHash = "$Bin/../var/recordings.hash";
my $g_RecDir        = $g_RbcConf{'RecDir'} || '/media/reel/recordings/';
my %g_RbcStatus;
my $g_Debug;
my %g_Config;

## init signal exit_handler
$SIG{'KILL'} = \&exit_handler;
$SIG{'INT'}  = \&exit_handler;
$SIG{'TERM'} = \&exit_handler;

## buffer off
$| = 1;
select(*STDERR);
$| = 1;
select(*STDOUT);

#//////////////////////////////////////////////////////////////////////////////
# function forward declaration
sub GetRecords;
sub GetRecordDetail;
sub CheckUpdateFlag;
sub GetArgs;
sub ArgHelp;
sub exit_handler;
sub main;
sub GetIMDBCover;

#//////////////////////////////////////////////////////////////////////////////
# funtions
sub GetArgs {
  my $help;

  GetOptions(
    "--debug"       => \$g_Debug,
    "--force-load"  => \$g_Config{'force-load'},
    "--force-cover" => \$g_Config{'force-cover'},
    "--nocover"     => \$g_Config{'nocover'},
    "--help"        => \$help
  );

  if ( $help ) {
    ArgHelp;
    exit 0;
  }

}
#//////////////////////////////////////////////////////////////////////////////
sub ArgHelp {
  print <<EOF
Usage: $0 [OPTION]...
generate vdr recording index for further use in RBC

  --debug             ; enable debug output
  --force-load        ; force index (obsolete)
  --nocover           ; dont get any covers
  --force-cover       ; search cover again even not yet found
  --help              ; print this help

EOF
}
#//////////////////////////////////////////////////////////////////////////////
sub GetRecords {
  my $new_update_time = shift;
  my %NewRec;
  my %Records;
  my @OUT;
  my $initial;

  ## get rbc status
  if ( -e $g_RbcRecordHash ) {
    my $href;
    if ( eval { $href = retrieve( $g_RbcRecordHash ) } ) {
      print "load record hash ok\n" if $g_Debug;
      %Records = %{$href};
    }
    else {
      unlink($g_RbcRecordHash);
      print "load record hash failed! Unlink it ...\n" if $g_Debug;
    }
  }
  else {
    $initial = 'true';
  }

  my $RecID = 0;
  open( FIND, "find $g_RecDir -follow -name \"info.vdr\" |grep \".rec/\" |" )
    or die "$!";
  while (<FIND>) {
    chomp;
    $RecID++;
    my $key = $_;
    (my $path = $_) =~ s/$g_RecDir//;

    print "REC: $_\n" if $g_Debug;
    my $href = GetRecordDetail($_);

    ## get path info
    my $cntr = 0;
    foreach ( split( /\//, $path ) ) {
      my $dir = undef;
      $cntr++;

      $dir = 1 if( /\.rec$/ );

      next if ( /^info.vdr$/);
      $NewRec{$key}{'first'} = "false";

      if($cntr == 1) {
        $NewRec{$key}{'path'} = $path;
        $NewRec{$key}{'apath'}[$cntr] = $_ . "/";
      }
      elsif($cntr == 2) {
        $NewRec{$key}{'apath'}[$cntr] =
          $NewRec{$key}{'apath'}[$cntr-1] . $_ . "/";

        $NewRec{$key}{'first'}        = "true" if defined $dir;

        $NewRec{$key}{'recpath'}      =
          $NewRec{$key}{'apath'}[$cntr] if defined $dir;
      }
      else {
        $NewRec{$key}{'apath'}[$cntr] =
          $NewRec{$key}{'apath'}[$cntr-1] . $_ . "/";
        $NewRec{$key}{'recpath'} = $NewRec{$key}{'apath'}[$cntr-1] if defined $dir;
      }
    }

    ## check details
    if ( ref($href) ne "HASH" ) {
      print STDERR "error get detail[$RecID]: $href\n" if $g_Debug;
      $NewRec{$key}{'status'}      = 'error_get_detail';
      $NewRec{$key}{'status_text'} = $href;
      next;
    }

    ## save records
    $NewRec{$key}{detail}             = \%{$href};
    $NewRec{$key}{'detail'}{'RecID'}  = $RecID;
    $NewRec{$key}{'RecID'}            = $RecID;
    $NewRec{$key}{'titel'}            = $NewRec{$key}{'detail'}{'titel'};
    $NewRec{$key}{'esc_titel'}        = $NewRec{$key}{'detail'}{'esc_titel'};
    $NewRec{$key}{'status'}           = 'loaded';
    $NewRec{$key}{'recdir'}           = dirname($key);
    $NewRec{$key}{'detail'}{'recdir'} = dirname($key);
    ( $NewRec{$key}{'jsrecdir'} = $NewRec{$key}{'recdir'} ) =~ s/#/___H___/g;
    $NewRec{$key}{'jsrecdir'}   =~ s/'/___A___/g;
    $NewRec{$key}{'jsrecdir'}   =~ s/&/___U___/g;
    $NewRec{$key}{'detail'}{'jsrecdir'} = $NewRec{$key}{'jsrecdir'};

    ## check resume status
    if ( -e "$NewRec{$key}{'recdir'}/resume.vdr" ) {
      $NewRec{$key}{'resume'} = 1;
      $NewRec{$key}{'detail'}{'resume'} = 1;
    }
    else {
      $NewRec{$key}{'resume'} = undef;
      $NewRec{$key}{'detail'}{'resume'} = undef;
    }

    ## initial run. save every record to get records as soon as possible
    store \%NewRec, $g_RbcRecordHash if defined $initial;
  }
  close(FIND);

  ## save record hash
  store \%NewRec, $g_RbcRecordHash;

  return 'done';
}

#//////////////////////////////////////////////////////////////////////////////
sub GetRecordDetail {
  my $RecDir = shift;
  my %Detail;

  ## init structure
  $Detail{'audio'} = [];

  open( VDR, "< $RecDir" ) or die "$!";
  while (<VDR>) {
    chomp;
    s/^215-//;

    $Detail{'channel'}   = $1 if (/^C\s(.+\s*)$/);
    $Detail{'timestamp'} = $1 if (/^E\s(.+\s*)$/);
    $Detail{'titel'}     = $1 if (/^T\s(.+\s*)$/);
    $Detail{'subtitel'}  = $1 if (/^S\s(.+\s*)$/);
    $Detail{'descr'}     = $1 if (/^D\s(.+\s*)$/);
    $Detail{'vps'}       = $1 if (/^V\s(.+\s*)$/);
    $Detail{'epg'}       = $1 if (/^@\s(.+\s*)$/);
    $Detail{'hd'}        = 'true' if (/^X\s(.+HD-Video\s*)$/);

    push(@{$Detail{'audio'}} , $1) if (/^X\s(.+\s*)$/);

    $Detail{'esc_titel'} = uri_escape( $Detail{'titel'} );

    ## set time format
    if ( defined $Detail{'timestamp'} ) {
      if ( $Detail{'timestamp'} =~ /^(\d+) (\d{10}) (\d+?) (.*)$/ ) {
        my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) =
          localtime($2);

        $sec  = sprintf( "%02d", $sec );
        $min  = sprintf( "%02d", $min );
        $mon  = sprintf( "%02d", $mon + 1 );
        $mday = sprintf( "%02d", $mday );

        $Detail{'time'} = "$mday.$mon." . ( int($year) + 1900 ) . " $hour:$min";

        $hour = sprintf( "%02d", $hour );
        $Detail{'timeindex'} =
          ( int($year) + 1900 ) . $mon . $mday . $hour . $min . $sec;

        ## record length
        $Detail{'seconds'} = $3 if defined $3;
        $Detail{'length'}  = int( $3 / 60 ) if defined $3;
        print "set length: $Detail{'length'} \n" if $g_Debug;

      }
    }

    ## try to get the year
    if ( defined $Detail{'descr'} ) {
      $Detail{'year'} = $1 if /|\S{2-3} ([12][09]\d{2})\s*$/;
      $Detail{'year'} = $1 if /|\S{2-3} ([12][09]\d{2})\s*\./;
      $Detail{'year'} = $1 if /([12][09]\d{2})\s*$/;

      print "SET YEAR: $Detail{'year'}\n" if $Detail{'year'} && $g_Debug;
    }

    ## <channel>23 - kabel eins</channel>
    if ( defined $Detail{'epg'} ) {
      ( $Detail{'channel_name'} = $Detail{'epg'} ) =~
        s/^.*<channel>(.*)<\/channel>.*$/$1/;
    }

    print "DETAIL: $_\n" if $g_Debug;

  }
  close(VDR);

  return \%Detail;
}

#//////////////////////////////////////////////////////////////////////////////
sub GetIMDBCover {
  my $search  = shift;
  my $rec_dir = shift;
  my %REC = %{ retrieve($g_RbcRecordHash) };
  my @IMDB_URLS_REF = ( 'www.imdb.de', 'www.imdb.com' );
  my @IMDB_URLS = @IMDB_URLS_REF;

  use Proc::PID::File;

  ## check if another index job already running
  if ( Proc::PID::File->running() ) {
    print "INDEX COVER already running\n";
    return;
  }

  ## check force cover flagfile
  if ( -e '/tmp/cover.force' ) {
    print "found cover force flag. Enable --force-cover\n" if $g_Debug;
    unlink( '/tmp/cover.force' );
    $g_Config{'force-cover'} = 1;
  }

  foreach my $r ( keys %REC ) {

    ## dir writable check
    next if ! -w $REC{$r}{'recdir'};

    next if ! $REC{$r}{titel};
    ## trim subtitel to get a better search result
    ## $REC{$_}{titel} =~ s/\s*- .+$//;
    $REC{$r}{titel} =~ s/\s*-\s*Director.*?Cut\s*$//i;
    $REC{$r}{titel} =~ s/\s*-\s*Extended\s*Version\s*$//i;
    $REC{$r}{titel} =~ s/\(.+\)//;
    $REC{$r}{titel} =~ s/\s*$//;

    print "COVER: get '$REC{$r}{titel}'\n" if $g_Debug;

    if ( -e "$REC{$r}{recdir}/cover.notfound"
      && !defined $g_Config{'force-cover'} )
    {
      print "COVER: already run -> not found '$REC{$r}{titel}'\n" if $g_Debug;
      @IMDB_URLS = @IMDB_URLS_REF;
      next;
    }

    if ( -e "$REC{$r}{recdir}/cover.jpg" ) {
      print "COVER: already done\n" if $g_Debug;
      @IMDB_URLS = @IMDB_URLS_REF;
      next;
    }

    my $film;

    print "HOST: $IMDB_URLS[0]\n" if $g_Debug;

    eval {
      $film = new IMDB::Film(
                crit => uri_escape( $REC{$r}{titel} ),
                host => $IMDB_URLS[0]
              );
    };

    if ( ! defined $film || ! $film->status() ) {
      print "COVER: not found - $REC{$r}{titel}\n" if $g_Debug;

      ## cut - subtitel
      redo if $REC{$r}{titel} =~ s/\s+- .+$//;

      ## next imdb server
      shift @IMDB_URLS;
      if ( @IMDB_URLS < 1 ) {
        touch("$REC{$r}{recdir}/cover.notfound");
        @IMDB_URLS = @IMDB_URLS_REF;
        next;
      }
      print "REDO: $IMDB_URLS[0]\n" if $g_Debug;
      redo;
    }

    my $title = $film->title();
    my $file  = $REC{$r}{recdir} . '/cover.jpg';

    if ( ! $film->cover() ) {
      print "COVER: titel found -- cover not found - $REC{$r}{titel}\n" if $g_Debug;


      ## try to get the cover from imdb.com
      my @MATCHES = @{ $film->matched };

      if ( $film->code() or ( @MATCHES > 0 && ref( $MATCHES[0] ) eq 'HASH' ) ) {
        my $id = $film->code() || @{ $film->matched }[0]->{id};
        print "CODE: $id\n" if $g_Debug;
        $film = undef;
        eval { $film = new IMDB::Film(crit => $id, host => 'www.imdb.com' ) if $id; };
        if( $film && $film->status && $film->cover ) {
          my $dur = $film->duration();
          $dur = $1 if $dur && $dur =~ /^.*?(\d+) min.*$/i;

          # if ( $REC{$r}{'detail'}{'year'} && defined $film->year
          #       && $REC{$r}{'detail'}{'year'}  != $film->year )
          # {
          #  print "YEAR MISSMATCH: " , $film->title , " not equal $REC{$r}{titel}\n" if $g_Debug;
          # }
          if ( $REC{$r}{'detail'} && $REC{$r}{'detail'}{'length'}
                 && $dur
                 && 1.3 * ( $REC{$r}{'detail'}{'length'} + 10 ) <  int( $dur ) )
          {
            print "LENGTH MISSMATCH: $dur not equal $REC{$r}{'detail'}{'length'} "
              , $film->title, "\n" if $g_Debug;
          }
          {
            print "OVERWRITE: " , $film->title , "\n" if $g_Debug;
            my $cover = $film->cover();
            getstore($cover,$file);
            @IMDB_URLS = @IMDB_URLS_REF;
            next;
          }
        }
      }

      ## cut - subtitel
      redo if $REC{$r}{titel} =~ s/\s+- .+$//;

      shift @IMDB_URLS;

      if ( @IMDB_URLS < 1 ) {
        touch("$REC{$r}{recdir}/cover.notfound");
        @IMDB_URLS = @IMDB_URLS_REF;
        next;
      }
      print "REDO: $IMDB_URLS[0]\n" if $g_Debug;
      redo;
    }

    my $cover = $film->cover();
    getstore($cover,$file);
    @IMDB_URLS = @IMDB_URLS_REF;
    print "Done\n" if $g_Debug;
  }

}
#//////////////////////////////////////////////////////////////////////////////
sub exit_handler {
  my $signal = shift;    # signal-nummer besorgen

  ## try to unlock
  # unlock($g_LockFile);

  unlock('/tmp/rbc');

  exit 1;
}

#//////////////////////////////////////////////////////////////////////////////
sub CheckUpdateFlag {
  my $last_update = shift || 0;

  # hack
  my $file  = `ls -1tr $g_RecDir|tail -1`;
  my $count = `find $g_RecDir/ -name info.vdr|grep ".rec/"| wc -l`;
  return if ( !defined $file );
  chomp($file);
  chomp($count);

  my $st = stat("$g_RecDir/$file") or die "Couldn't stat $g_RecDir/$file: $!";
  my $mtime = $st->mtime;

  $mtime .= $count;

  print "SET TIME $mtime\n" if $g_Debug;

  if ( $last_update == $mtime ) {
    return undef;
  }
  else {
    print "Time: $mtime\n";
  }

  return $mtime;

}

#//////////// //////////////////////////////////////////////////////////////////
# program starts here
sub main {

  ## init
  GetArgs;

  ##  check vdrstatus
  # if(! vdr_status && ! $g_Config{'force-load'}) {
  #   print "ReelVDR not running!\n" if $g_Debug;
  #  exit 0;
  # }

  ## lock
  #if(! lock('/tmp/rbc')) {
  #  print "cat't lock!\n";
  #  exit 1;
  #}

  ## get rbc status
  if ( -e $g_RbcStatusHash ) {
    my $href = retrieve($g_RbcStatusHash);
    if ( ref($href) eq "HASH" ) {
      %g_RbcStatus = %{$href};
    }
    else {
      unlink($g_RbcStatusHash);
    }
    ## force-load
    $g_RbcStatus{'last_update'} = 1 if ( defined $g_Config{'force-load'} );

  }
  else {
    $g_RbcStatus{'last_update'} = 1;
  }

  my $new_update_time = CheckUpdateFlag;

  ## read record overview
  if ( GetRecords($new_update_time) eq 'done' ) {
    ## save new update timestamp
    $g_RbcStatus{'last_update'} = $new_update_time;
  }

  ## save RbStatus
  store \%g_RbcStatus, $g_RbcStatusHash;

  ## try unlock
  #unlock('/tmp/rbc');

  GetIMDBCover if ! defined $g_Config{'nocover'};

  exit 0;
}
main;
print STDERR "unexpected end of script\n";
exit_handler;

#//////////////////////////////////////////////////////////////////////////////
END {
  ## unlock
}
