package ReelBox::Base;

use strict;
use warnings;
use File::stat;
use Socket;

require Exporter;
our @ISA       = qw(Exporter);
our @EXPORT_OK = qw( vdr_status get_config get_bookmark svdrp read_file );

my %CONF = %{ get_config('/opt/rbc/htdocs/config/reelbox.cfg') };
my $g_svdrpsend = '/opt/rbc/bin/svdrpsend.sh';
my $g_pid_file  = '/var/run/reelvdrd.pid';

## function forward declaration
sub _svdrp_receive;
sub _svdrp_send;

#///////////////////////////////////////////////////////////////////////////////
# check vdr is running
sub vdr_status {

  ## check forceflag
  return 1 if -e '/tmp/vdr.forcerunning';

  system("ps -ef|grep -v grep| grep -c /usr/sbin/vdr >/dev/null 2>&1");

  if ( $? == 0 ) {
    return 1;
  }

  return undef;

  ## hmm not the vdr pid file ...
  open( P, "< $g_pid_file");
    my $pid =  <P>;
  close(P);

  my $running = kill 0, $pid;

  return $running;
}

sub standby {
  svdrp( 'HITK Power' );

  # wait until vdr shutdown
  # try 30 time and give up.
  # TODO: write back message, if shutdown failed
  for ( my $i = 0 ; $i <= 30 ; $i++ ) {
    last if ! vdr_status;
    sleep 1;
  }
}

sub wakeup {
  unlink "/tmp/vdr.standby";
  unlink "/tmp/vdr.shutdown";

  # wait for startup
  # try 20 time and give up.
  # TODO: write back message, if startup failed
  #
  for ( my $i = 0 ; $i <= 20 ; $i++ ) {
    last if vdr_status;
    sleep 1;
  }
}

sub shutdown {


}

sub reboot {

}

sub rec_status {
  return undef if ! -e ( '/tmp/vdr.records' );

  open ( F, "< /tmp/vdr.records");
    my $count = <F>;
  close(F);

  return ( $count && int( $count ) > 0 ) ? $count : undef;
}
#///////////////////////////////////////////////////////////////////////////////
#
sub get_config {
  my $file = shift;
  my %CONF;

  if ( !-e $file ) { return undef;}
  open ( FILE, "< $file" );
    local $/;
    my $line =  <FILE>;
    $line =~ s/\r//g;
    %CONF = map { /^\s*([^#]+)=(.*)\s*$/ } split(/\n/, $line);
  close( FILE );

  return \%CONF;
}
#///////////////////////////////////////////////////////////////////////////////
# read bookmarkfile
sub get_bookmark {
  my $file = shift;
  my %CONF;

  if ( !-e $file ) { return undef;}
  open ( FILE, "< $file" );
    local $/;
    my $line =  <FILE>;
    $line =~ s/\r//g;
    %CONF = map { /^\s*([^#]+)?[\s\t]+(.*)?\s*$/ } split(/\n/, $line);
  close( FILE );

  return \%CONF;
}
#///////////////////////////////////////////////////////////////////////////////
#
sub GetLastRecordDate {

}
#///////////////////////////////////////////////////////////////////////////////
#
sub svdrp {
  my $cmd = shift;
  my @return;

  my $status = eval {
    my $timeout = 20;
    $SIG{ALRM} = sub { return "timeout"; };
    alarm($timeout);

    my $iaddr = inet_aton( $CONF{'ip_address'} )
      || return "no host: $CONF{'ip_address'}";
    my $paddr = sockaddr_in( '2001', $iaddr );
    my $proto = getprotobyname('tcp');

    socket( SOCK, PF_INET, SOCK_STREAM, $proto ) || return "socket: $!";
    connect( SOCK, $paddr ) || return "connect: $!";
    select(SOCK);
    $| = 1;              # buffer off
    select(STDOUT);
    _svdrp_receive;
    @return = _svdrp_send($cmd);
    _svdrp_send("quit");
    close(SOCK) || return "close: $!";
    return undef;
  };


  return wantarray ? @return : undef, $status;
}
#///////////////////////////////////////////////////////////////////////////////
#
sub _svdrp_send {
  my $cmd = shift || return 'no command to send';
  print SOCK "$cmd\r\n";
  _svdrp_receive;

}
#///////////////////////////////////////////////////////////////////////////////
#
sub _svdrp_receive {
  my @return;
  while (<SOCK>) {
    push(@return,$_);
    last if substr($_, 3, 1) ne '-';
  }
  return @return;
}

sub read_file {
  my $file = shift;
  my $m = shift;
  if ( !-e $file ) {

  }
  else {
    my $line;
    open ( FILE, "$file") or $m->comp('SELF:error', error => "file open failed: $!" );
      local $/ = undef;
      $line    = <FILE>;
    close(FILE);
    return wantarray ? split("\n", $line) : $line;
  }
}
#///////////////////////////////////////////////////////////////////////////////
1;
